import 'package:flutter/material.dart';
import 'package:hub/backend/schema/enums/enums.dart';

import '/components/organism_components/menu_list_view_component/menu_list_view_component_widget.dart';
import '/components/organism_components/menu_staggered_view_component/menu_staggered_view_component_widget.dart';
import '/flutter_flow/flutter_flow_util.dart';
import 'menu_component_model.dart';

export 'menu_component_model.dart';

class MenuComponentWidget extends StatefulWidget {
  final MenuComponentModel model;
  const MenuComponentWidget({super.key, required this.model});

  @override
  State<MenuComponentWidget> createState() => _MenuComponentWidgetState();
}

class _MenuComponentWidgetState extends State<MenuComponentWidget> {
  @override
  void setState(VoidCallback callback) {
    super.setState(callback);
    widget.model.onUpdate();
  }

  @override
  void initState() {
    super.initState();
    widget.model.safeSetState = () => safeSetState(() {});
  }

  @override
  void didChangeDependencies() async {
    super.didChangeDependencies();
    widget.model.menuEntries =
        await widget.model.generateMenuEntries(context, widget.model.item, widget.model.menuOptions);
  }

  @override
  void dispose() {
    widget.model.maybeDispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Builder(
      builder: (context) {
        if (widget.model.style == MenuView.list_grid)
          return wrapWithModel(
            model: widget.model.menuStaggeredViewComponentModel,
            updateCallback: () => setState(() {}),
            updateOnChange: true,
            child: MenuStaggeredViewComponentWidget(
              options: widget.model.menuEntries,
              expandable: widget.model.expandable,
              item: widget.model.item,
              changeMenuStyle: () async {},
              isGrid: widget.model.isGrid,
            ),
          );
        if (widget.model.style == MenuView.list)
          return wrapWithModel(
            model: widget.model.menuListViewComponentModel,
            updateCallback: () => setState(() {}),
            updateOnChange: true,
            child: MenuListViewComponentWidget(
              options: widget.model.menuEntries,
              expandable: widget.model.expandable,
              item: widget.model.item,
              changeMenuStyle: () async {},
            ),
          );

        return const SizedBox();
      },
    );
  }
}
