import 'package:hub/shared/helpers/license/license_helper.dart';
import 'package:hub/shared/helpers/storage/keychain_storage.dart';
import 'package:hub/shared/services/license/license_service.dart';
import 'package:sqflite/sqflite.dart';
import 'package:path/path.dart';
import 'dart:developer';

class DatabaseStorage {
  static final String _dbName = 'database.db';
  static final int _dbVersion = 2;

  static final DatabaseStorage instance = DatabaseStorage._internal();
  factory DatabaseStorage() => instance;
  DatabaseStorage._internal();

  static Database? _database;
  static Database get database {
    if (_database == null) {
      throw Exception('Database has not been initialized. Call init() first.');
    }
    return _database!;
  }

  Future<void> init() async {
    if (_database != null) return;
    _database = await openDatabase(
      join(await getDatabasesPath(), _dbName),
      version: _dbVersion,
      onCreate: _onCreate,
      onUpgrade: _onUpgrade,
      onDowngrade: _onDowngrade,
    );
    await LicenseService.setupLicense(database);
  }

  Future<void> _onCreate(Database database, int version) async {
    log('Creating database...');
    await database.execute(KeychainHelper.createTableQuery);
    await database.execute(LicenseHelper.createTableQuery);
  }

  Future<void> _onUpgrade(Database database, int oldVersion, int newVersion) async {
    log('Upgrading database from version $oldVersion to $newVersion...');
    if (oldVersion < 2) {
      await database.execute(LicenseHelper.createTableQuery);
      await LicenseService.setupLicense(database);
    }
  }

  Future<void> _onDowngrade(Database database, int oldVersion, int newVersion) async {}
}
